Package: greeks
Title: Sensitivities of Prices of Financial Options and Implied
        Volatilities
Version: 1.5.6
Authors@R: 
    person(given = "Anselm",
           family = "Hudde",
           role = c("aut", "cre"), 
           email = "anselmhudde@gmx.de",
           comment = c(ORCID = "0000-0002-5652-2815"))
Description: Methods to calculate sensitivities of financial option prices for
 European, geometric and arithmetic Asian, and American options, with various
 payoff functions in the Black Scholes model, and in more general jump diffusion
 models. A shiny app to interactively plot the results is included. Furthermore,
 methods to compute implied volatilities are provided for a wide range of option
 types and  custom payoff functions. Classical formulas are implemented for
 European options in the Black Scholes Model, as is presented in Hull, J. C.
 (2017), Options, Futures, and Other Derivatives.
 In the case of Asian options, Malliavin Monte Carlo Greeks are implemented, see
 Hudde, A. & Rüschendorf, L. (2023). European and Asian Greeks for exponential
 Lévy processes. <doi:10.1007/s11009-023-10014-5>. For American
 options, the Binomial Tree  Method is implemented, as is presented in Hull,
 J. C. (2017). 
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), R.rsp
Config/testthat/edition: 3
Imports: magrittr, dqrng, Rcpp, tibble, ggplot2, plotly, shiny, tidyr
LinkingTo: Rcpp
URL: https://github.com/ahudde/greeks
BugReports: https://github.com/ahudde/greeks/issues
NeedsCompilation: yes
Packaged: 2026-07-17 13:19:40 UTC; ahudde
Author: Anselm Hudde [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5652-2815>)
Maintainer: Anselm Hudde <anselmhudde@gmx.de>
VignetteBuilder: knitr
Repository: CRAN
Date/Publication: 2026-07-17 13:40:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-17 16:53:52 UTC; unix
Archs: greeks.so.dSYM
