# Generated by roxygen2: do not edit by hand

export("%>%")
export(BS_European_Greeks)
export(BS_Geometric_Asian_Greeks)
export(BS_Implied_Volatility)
export(BS_Malliavin_Asian_Greeks)
export(Binomial_American_Greeks)
export(Greeks)
export(Greeks_UI)
export(Implied_Volatility)
export(Malliavin_Asian_Greeks)
export(Malliavin_European_Greeks)
export(Malliavin_Geometric_Asian_Greeks)
import("Rcpp")
import("ggplot2")
import("shiny")
import("stats")
import("tibble")
import("tidyr")
importFrom("dqrng","dqrnorm")
importFrom("dqrng","dqset.seed")
importFrom("plotly","ggplotly")
importFrom("plotly","plotlyOutput")
importFrom("plotly","renderPlotly")
importFrom(magrittr,"%>%")
useDynLib(greeks, .registration=TRUE)
