# greeks
derivatives

[![Build Status](https://travis-ci.org/timkpaine/greeks.svg?branch=master)](https://travis-ci.org/timkpaine/greeks)
[![GitHub issues](https://img.shields.io/github/issues/timkpaine/greeks.svg)]()
[![codecov](https://codecov.io/gh/timkpaine/greeks/branch/master/graph/badge.svg)](https://codecov.io/gh/timkpaine/greeks)
[![BCH compliance](https://bettercodehub.com/edge/badge/timkpaine/greeks?branch=master)](https://bettercodehub.com/)
[![PyPI](https://img.shields.io/pypi/l/greeks.svg)](https://pypi.python.org/pypi/greeks)
[![PyPI](https://img.shields.io/pypi/v/greeks.svg)](https://pypi.python.org/pypi/greeks)
[![Docs](https://img.shields.io/readthedocs/greeks.svg)](https://greeks.readthedocs.io)
